#!/usr/bin/bash

export PYTHONDONTWRITEBYTECODE=1

# set to 1 for debugging
export EXTRACT_PKG="0"

if [[ -z $VERSION ]]; then
  echo "Warning, setting VERSION env var to default value 0.0.1-0"
  export VERSION="0.0.1-0"
fi
echo "Building Package version: $VERSION"

# BE CAREFUL TO NOT BUILD IN A PYTHON VENV!

# be sure to change version if needed!
fpm -s python --python-bin python3 --python-package-name-prefix python3 \
    -m '"Joel Klimont" <joel.klimont@comp-air.at>' \
    --license 'proprietary' \
    --description 'Library for robot used in the competition' \
    --after-install postinstall.sh \
    --after-upgrade postinstall.sh \
    --deb-priority "optional" \
    --architecture "aarch64" \
    -d "python3-pip" \
    -d "nginx" \
    -d "libnginx-mod-rtmp" \
    -d "libatlas-base-dev" \
    -d "python3-numpy" \
    -d "opencv-dev" \
    -d "opencv-libs" \
    -d "opencv-licenses" \
    -d "opencv-main" \
    -d "opencv-python" \
    -d "opencv-scripts" \
    -d "libprotobuf23" \
    -d "protobuf-compiler" \
    -d "python3-protobuf" \
    --python-install-lib "/usr/local/lib/python3.9/dist-packages" \
    -v $VERSION -t deb setup.py

if [ "$EXTRACT_PKG" == "1" ]; then
  echo "Extracting deb package"
  mkdir build_extract
  mv python3-complib_"$VERSION"_arm64.deb build_extract
  cd build_extract
  ar -xv python3-complib_"$VERSION"_arm64.deb
fi

export DEB=python3-complib_"$VERSION"_arm64.deb
echo "Created: $DEB"

#    --deb-changelog changelog \
#    --deb-upstream-changelog changelog \
#    --deb-field "Distribution: stable" \
#    --deb-dist "stable" \

# sudo apt purge python3-complib -y
# sudo apt install ./python3-*
# sudo apt search complib
# ar vx ./python3*
